##
# File:  DepictContent.py
# Date:  30-Mar-2016
#
# Updates:
#  09-Dec-2024  zf   call _getPdbExtIdMap() method to get 'ext_pdb_id'
#
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys
try:
    from urllib.parse import quote as u_quote
except ImportError:
    from urllib import quote as u_quote

from wwpdb.apps.workmanager.depict.DepictBase import DepictBase, processPublicIDs
from wwpdb.apps.workmanager.depict.ReadConFigFile import dumpPickleFile, loadPickleFile


class DepictContent(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(DepictContent, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        self.__returnMap = {}
        self._connectStatusDB()
        self._getUserInfoDict()
        self.__commun_tmplt = self._getPageTemplate('commun_tmplt')
        self.__commun_image_tmplt = self._getPageTemplate('commun_image_tmplt')
        self.__assign_annotator_tmplt = ''
        self.__AnnotatorSelection = ''
        self.__dataFieldMap = {}
        if 'table_data_field_binding' in self._conFigObj:
            self.__dataFieldMap = self._conFigObj['table_data_field_binding']
        #
        self.__sObj = self._reqObj.newSessionObj()
        self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()

    def depictTableContent(self, index):
        """
        """
        self.__returnMap = {}
        tableContentMap = loadPickleFile(self.__sessionPath, 'TableContentMap.pkl')
        if index == 'all':
            for tableMap in tableContentMap.values():
                self._depictTableContent(tableMap)
            #
        elif index in tableContentMap:
            self._depictTableContent(tableContentMap[index])
        #
        return self.__returnMap

    def _depictTableContent(self, tableMap):
        """
        """
        if (not tableMap) or ('pkl' not in tableMap):
            return
        #
        if 'binding_function' in tableMap:
            if 'binding_class' in tableMap:
                self._processBindingClass(tableMap['binding_class'])
                classUtil = self._UtilClass[tableMap['binding_class']]
                _columnDef, contentResults = getattr(classUtil, '%s' % tableMap['binding_function'])()
            else:
                _columnDef, contentResults = getattr(self, '%s' % tableMap['binding_function'])()
            #
            dumpPickleFile(self.__sessionPath, tableMap['pkl'], contentResults)
            return
        #
        if ('sql' not in tableMap) or ('data-field' not in tableMap):
            return
        #
        count_map = self.__initializeStatusCount(tableMap)
        contentResults = []
        rows = self._statusDB.runSelectSQL(tableMap['sql'])
        if rows and ('sort_function' in tableMap):
            rows = getattr(self, '%s' % tableMap['sort_function'])(rows)
        #
        if rows:
            pdbExtIdMap = {}
            if ('pdb_ids' in tableMap['data-field']) or ('user_pdb_id' in tableMap['data-field']):
                pdbExtIdMap = self._getPdbExtIdMap(rows)
            #
            groupIdMap = {}
            idList = self.__getEntryIDList(rows, 'D_')
            if idList:
                return_list = self._statusDB.getGroupIds(depositionids=idList)
                if return_list:
                    for rD in return_list:
                        if ('group_id' in rD) and rD['group_id'] and ('dep_set_id' in rD) and rD['dep_set_id']:
                            groupIdMap[rD['dep_set_id']] = rD['group_id']
                        #
                    #
                #
                foundCombDateItem = False
                for field_item in tableMap['data-field']:
                    if field_item not in self.__dataFieldMap:
                        continue
                    #
                    value = self.__dataFieldMap[field_item]['value']
                    if value == 'deposition_release_dates':
                        foundCombDateItem = True
                        break
                    #
                #
                if foundCombDateItem:
                    self._connectContentDB()
                    releaseDateMap = self._contentDB.getReleaseDate("', '".join(sorted(set(idList))))
                    for dataD in rows:
                        dataD['deposition_release_dates'] = ''
                        if ('dep_set_id' not in dataD) or (not dataD['dep_set_id']):
                            continue
                        #
                        if ('dep_initial_deposition_date' in dataD) and dataD['dep_initial_deposition_date']:
                            dataD['deposition_release_dates'] = str(dataD['dep_initial_deposition_date']) + ' /<br /> '
                            if (dataD['dep_set_id'] in releaseDateMap) and releaseDateMap[dataD['dep_set_id']]:
                                dataD['deposition_release_dates'] += releaseDateMap[dataD['dep_set_id']]
                            else:
                                dataD['deposition_release_dates'] += 'n.a.'
                            #
                        elif (dataD['dep_set_id'] in releaseDateMap) and releaseDateMap[dataD['dep_set_id']]:
                            dataD['deposition_release_dates'] = 'n.a. /<br /> ' + releaseDateMap[dataD['dep_set_id']]
                        #
                    #
                #
            #
            annSelectMap = {}
            reminderSentMap = {}
            PIInfoMap = {}
            # if ('add_list' in tableMap['data-field']) or ('major_issue' in tableMap['data-field']) or \
            #        ('pi_name' in tableMap['data-field']) or ('country' in tableMap['data-field']) or \
            #        ('pi_name_only' in tableMap['data-field']) or ('pi_country_only' in tableMap['data-field']):

            if (  # pylint: disable=using-constant-test
                    x for x in tableMap['data-field'] if x in ('add_list',
                                                               'major_issue',
                                                               'pi_name',
                                                               'country',
                                                               'pi_name_only',
                                                               'pi_country_only',
                                                               'received_date')
            ):
                if idList and ('add_list' in tableMap['data-field']):
                    return_list = self._statusDB.getAnnoSelection(depositionids=idList)
                    annSelectMap = self.__convertListIntoMap(return_list)
                #
                if idList and (x for x in tableMap['data-field'] if x in ('major_issue',
                                                                          'received_date')
                               ):
                    return_list = self._statusDB.getRemindMessageTrack(depositionids=idList)
                    reminderSentMap = self.__convertListIntoMap(return_list)
                #
                if idList and (x for x in tableMap['data-field'] if x in ('pi_name',
                                                                          'country',
                                                                          'pi_name_only',
                                                                          'pi_country_only')
                               ):
                    PIInfoMap = self.__getPIInfo(idList)
                #
            #
            if 'assign_annotator' in tableMap['data-field']:
                self.__assign_annotator_tmplt = self._getPageTemplate('assign_annotator_tmplt')
                self.__AnnotatorSelection = self._getAnnotatorSelection()
            #
            order = 0
            for dataD in rows:
                dataD['display_ids'] = dataD['dep_set_id']
                dataD['group_info'] = ''
                if (dataD['dep_set_id'] in groupIdMap) and groupIdMap[dataD['dep_set_id']]:
                    dataD['display_ids'] = dataD['dep_set_id'] + '/' + groupIdMap[dataD['dep_set_id']]
                    dataD['group_info'] = '&group_id=' + groupIdMap[dataD['dep_set_id']]
                #
                if 'default_order' in tableMap['data-field']:
                    order_condition = ''
                    if 'order_condition' in tableMap:
                        order_condition = tableMap['order_condition']
                    #
                    dataD['default_order'] = self.__getOrder(order, len(rows), order_condition, dataD)
                #
                if count_map and ('dep_status_code' in dataD) and dataD['dep_status_code']:
                    status = str(dataD['dep_status_code'])
                    if status:
                        if status in count_map:
                            count_map[status] += 1
                        else:
                            count_map[status] = 1
                        #
                    #
                #
                dataD['locklabel'] = self.__processLockLabelForCommunication(dataD)
                if (dataD['dep_set_id'] in reminderSentMap) and reminderSentMap[dataD['dep_set_id']]:
                    dataD.update(reminderSentMap[dataD['dep_set_id']])
                #
                if (dataD['dep_set_id'] in PIInfoMap) and PIInfoMap[dataD['dep_set_id']]:
                    dataD.update(PIInfoMap[dataD['dep_set_id']])
                #
                if 'add_list' in tableMap['data-field']:
                    if (dataD['dep_set_id'] in annSelectMap) and annSelectMap[dataD['dep_set_id']]:
                        dataD['add_list'] = annSelectMap[dataD['dep_set_id']]['annotator_initials']
                    else:
                        dataD['add_list'] = 'Add'
                    #
                #
                dataD['base_url'] = ''
                if ('class_id' in dataD) and dataD['class_id']:
                    dataD['base_url'] = self._processBaseUrl(dataD['class_id'])
                #
                dataD['urlmethod'] = ''
                dataD['abbrv_method'] = ''
                if ('method' in dataD) and dataD['method']:
                    dataD['urlmethod'] = u_quote(dataD['method'])
                    if dataD['method'].upper() == 'X-RAY DIFFRACTION':
                        dataD['abbrv_method'] = 'X-RAY'
                    elif dataD['method'].upper() == 'NEUTRON DIFFRACTION':
                        dataD['abbrv_method'] = 'NEUTRON'
                    elif dataD['method'].upper() == 'FIBER DIFFRACTION' or dataD['method'].upper() == 'FIBRE DIFFRACTION':
                        dataD['abbrv_method'] = 'FIBER'
                    elif (
                            dataD['method'].upper() == 'CRYO-ELECTRON MICROSCOPY'
                            or dataD['method'].upper() == 'ELECTRON MICROSCOPY'
                            or dataD['method'].upper() == 'ELECTRON TOMOGRAPHY'
                    ):
                        dataD['abbrv_method'] = 'EM'
                    elif dataD['method'].upper() == 'ELECTRON CRYSTALLOGRAPHY':
                        dataD['abbrv_method'] = 'EL. CRYS.'
                    elif dataD['method'].upper() == 'SOLUTION NMR':
                        dataD['abbrv_method'] = 'NMR'
                    elif dataD['method'].upper() == 'SOLID-STATE NMR' or dataD['method'].upper() == 'SOLID STATE NMR':
                        dataD['abbrv_method'] = 'SS NMR'
                    else:
                        dataD['abbrv_method'] = dataD['method']
                    #
                #
                if ('pdb_ids' in tableMap['data-field']) or ('user_pdb_id' in tableMap['data-field']):
                    dataD = processPublicIDs(dataD, pdbExtIdMap)
                    if ('coor_status' in tableMap['data-field']) or ('author_status' in tableMap['data-field']):
                        dataD['comb_status_code'], dataD['comb_author_release_status_code'], titleEM, authorListEM = self.__processStatusCode(dataD)
                        if titleEM:
                            dataD['dep_title'] = titleEM
                        #
                        if authorListEM:
                            dataD['dep_author_list'] = authorListEM
                        #
                    #
                #
                self._dataInfo['data_for_all'] = [dataD]
                resultD = {}
                for field_item in tableMap['data-field']:
                    resultD[field_item] = ''
                    if field_item not in self.__dataFieldMap:
                        continue
                    #
                    value = self.__dataFieldMap[field_item]['value']
                    dtype = self.__dataFieldMap[field_item]['type']
                    if dtype == 'page_template':
                        resultD[field_item] = self.getPageText(page_id=value)
                    elif dtype == 'dataInfo':
                        if (value in dataD) and dataD[value]:
                            resultD[field_item] = str(dataD[value])
                        #
                    elif dtype == 'function':
                        resultD[field_item] = getattr(self, "%s" % value)(dataD)
                    #
                #
                contentResults.append(resultD)
                order += 1
            #
        #
        dumpPickleFile(self.__sessionPath, tableMap['pkl'], contentResults)
        #
        if 'entry_count' in tableMap:
            for t_type, ilist in tableMap['entry_count'].items():
                for item in ilist:
                    count = 0
                    if item == 'num_entries':
                        count = len(rows)
                    elif item in count_map:
                        count = count_map[item]
                    #
                    self.__returnMap[t_type + '_' + item + '_' + tableMap['tab_count_id']] = count
                #
            #
        #

    def _processWorkFlowStatus(self, dataD):
        """
        """
        if dataD['inst_status'] == 'waiting':
            return self.getPageText(page_id='workflow_waiting_status_tmplt')
        elif dataD['inst_status'] == 'closed(0)':
            return self.getPageText(page_id='workflow_close_status_tmplt')
        else:
            return self.getPageText(page_id='workflow_other_status_tmplt')

    def _processAssignSelection(self, dataD):
        """
        """
        myD = {}
        myD['dep_set_id'] = dataD['dep_set_id']
        myD['annotator_selection'] = self.__AnnotatorSelection
        return self.__assign_annotator_tmplt % myD

    def _processWorkFlowAction(self, dataD):
        """
        """
        if (dataD['dep_locking'].upper() == 'WFM') or (dataD['dep_status_code'].upper() == 'DEP'):
            return self.getPageText(page_id='workflow_lock_action_tmplt')
        elif dataD['inst_status'].lower() == 'init':
            return self.getPageText(page_id='workflow_init_action_tmplt')
        else:
            return self.getPageText(page_id='workflow_other_action_tmplt')
        #

    def _processAddList(self, dataD):
        """
        """
        if dataD['add_list'] == 'Add':
            return self.getPageText(page_id='add_list_tmplt')
        #
        return dataD['add_list']

    def _processCommunication(self, dataD):
        """
        """
        myD = {}
        myD['sessionid'] = self.__sessionId
        myD['initials'] = self._userInfo['initials']
        myD['dep_set_id'] = dataD['dep_set_id']
        myD['urlmethod'] = dataD['urlmethod']
        myD['locklabel'] = dataD['locklabel']
        #
        notify = dataD['dep_notify']
        imgD = {}
        imgD['image'] = 'wfm_comm.png'
        imgD['alt'] = 'Communication'
        if notify.find('N') != -1:
            imgD['image'] = 'wfm_new.png'
            imgD['alt'] = 'New Communication'
        elif notify.find('T') != -1:
            imgD['image'] = 'wfm_todo.png'
            imgD['alt'] = 'Communication to act on'
        #
        text = self.__commun_image_tmplt % imgD
        if notify.find('*') != -1:
            imgD['image'] = 'wfm_note.png'
            imgD['alt'] = 'Note'
            text += ' ' + self.__commun_image_tmplt % imgD
        #
        if notify.find('B') != -1:
            imgD['image'] = 'wfm_bmrb.png'
            imgD['alt'] = 'BMRB Message'
            text += ' ' + self.__commun_image_tmplt % imgD
        #
        if notify.find('A') != -1:
            imgD['image'] = 'wfm_approve.png'
            imgD['alt'] = 'Approve Message'
            text += ' ' + self.__commun_image_tmplt % imgD
        #
        myD['commun_image'] = text
        return self.__commun_tmplt % myD

    def _processAuxiliary(self, dataD):
        """
        """
        dep_notify = ''
        if 'dep_notify' in dataD:
            dep_notify = dataD['dep_notify']
        #
        dep_locking = ''
        if 'dep_locking' in dataD:
            dep_locking = dataD['dep_locking']
        #
        if dep_notify.find('R') != -1 and dep_locking == 'WFM':
            return 'background-red'
        elif dep_notify.find('R') != -1:
            return 'background-lightblue'
        elif dep_locking == 'WFM':
            return 'background-medpink'
        # If nothing else fall through
        # if dep_notify.find('A') != -1:
        #    return 'background-cyangreen'

        return ''

    def submit_group(self, rows):
        """ further process submitted group deposition
        """
        return self.__submit_group_filter(rows, True)

    def submit_group_search(self, rows):
        """ further process submitted group deposition
        """
        return self.__submit_group_filter(rows, False)

    def __submit_group_filter(self, rows, filter_by_annotator_flag):
        """
        """
        idList = self.__getEntryIDList(rows, 'G_')
        return_rows = self._statusDB.getEntryListForGroup(groupids=idList)
        groupIdMap = {}
        entryIdList = []
        for dataD in return_rows:
            if ('group_id' not in dataD) or (not dataD['group_id']) or ('dep_set_id' not in dataD) or (not dataD['dep_set_id']):
                continue
            #
            if dataD['group_id'] in groupIdMap:
                groupIdMap[dataD['group_id']].append(dataD['dep_set_id'])
            else:
                groupIdMap[dataD['group_id']] = [dataD['dep_set_id']]
                entryIdList.append(dataD['dep_set_id'])
            #
        #
        if not entryIdList:
            return []
        #
        info_rows = self._statusDB.getSimpleEntryInfo(depositionids=entryIdList)
        if not info_rows:
            return []
        #
        infoMap = {}
        for dataD in info_rows:
            infoMap[dataD['dep_set_id']] = dataD
        #
        groupInfoMap = {}
        for group_id, entry_ids in groupIdMap.items():
            if not entry_ids[0] in infoMap:
                continue
            #
            if (
                    ('annotator_initials' not in infoMap[entry_ids[0]]) or (not infoMap[entry_ids[0]]['annotator_initials'])
                    or (infoMap[entry_ids[0]]['annotator_initials'] != self._getUserInfo('initials'))) and filter_by_annotator_flag:
                continue
            #
            groupInfoMap[group_id] = {}
            groupInfoMap[group_id]['initial_deposition_date'] = ''
            if ('initial_deposition_date' in infoMap[entry_ids[0]]) and infoMap[entry_ids[0]]['initial_deposition_date']:
                groupInfoMap[group_id]['initial_deposition_date'] = infoMap[entry_ids[0]]['initial_deposition_date']
            #
            groupInfoMap[group_id]['status_code'] = self.__getGroupStatusCode(entry_ids)
        #
        group_rows = []
        for dataD in rows:
            if ('dep_set_id' not in dataD) or (not dataD['dep_set_id']) or (dataD['dep_set_id'] not in groupInfoMap):
                continue
            #
            for item in ('initial_deposition_date', 'status_code'):
                dataD[item] = groupInfoMap[dataD['dep_set_id']][item]
            #
            group_rows.append(dataD)
        #
        return group_rows

    def unsubmit_group(self, rows):
        """ further process un-submitted group deposition
        """
        nRows = []
        tRows = []
        cRows = []
        rRows = []
        for dataD in rows:
            if ('dep_notify' in dataD) and dataD['dep_notify']:
                if dataD['dep_notify'].find('N') != -1:
                    nRows.append(dataD)
                elif dataD['dep_notify'].find('T') != -1:
                    tRows.append(dataD)
                else:
                    cRows.append(dataD)
                #
            else:
                rRows.append(dataD)
            #
        #
        if tRows:
            nRows.extend(tRows)
        #
        if cRows:
            nRows.extend(cRows)
        #
        if rRows:
            nRows.extend(rRows)
        #
        return nRows

    def __initializeStatusCount(self, tableMap):
        """
        """
        count_map = {}
        if 'entry_count' in tableMap:
            for itemList in tableMap['entry_count'].values():
                for item in itemList:
                    if item == 'num_entries':
                        continue
                    #
                    count_map[item] = 0
                #
            #
        #
        return count_map

    def __getEntryIDList(self, rows, prefix):
        """
        """
        idlist = []
        for dataD in rows:
            if ('dep_set_id' in dataD) and dataD['dep_set_id'] and dataD['dep_set_id'].startswith(prefix):
                idlist.append(dataD['dep_set_id'])
            #
        #
        return idlist

    def __convertListIntoMap(self, rList):
        """
        """
        Map = {}
        if not rList:
            return Map
        #
        for dataD in rList:
            Map[dataD['dep_set_id']] = dataD
        #
        return Map

    def __getPIInfo(self, rList):
        """
        """
        Map = self.__getPIInfoFromCotentDB(rList)
        foundMissing = False
        for depId in rList:
            if depId not in Map:
                foundMissing = True
                break
            #
        #
        if not foundMissing:
            return Map
        #
        Map1 = self.__getPIInfoFromStatusDB(rList)
        for depId in rList:
            if depId in Map:
                continue
            #
            if depId in Map1:
                Map[depId] = Map1[depId]
            else:
                validContactAuthor = self._statusDB.ValidContactAuthor(depId)
                if validContactAuthor:
                    tmpMap = {}
                    if ('last_name' in validContactAuthor) and validContactAuthor['last_name']:
                        tmpMap['pi_name'] = validContactAuthor['last_name']
                    #
                    if ('country' in validContactAuthor) and validContactAuthor['country']:
                        tmpMap['country'] = validContactAuthor['country']
                    #
                    if tmpMap:
                        Map[depId] = tmpMap
                    #
                #
            #
        #
        return Map

    def __getPIInfoFromCotentDB(self, rList):
        """
        """
        self._connectContentDB()
        return self.__processPIInfo(self._contentDB.ContactAuthorPI(rList), ['name_first', 'name_mi', 'name_last'])

    def __getPIInfoFromStatusDB(self, rList):
        """
        """
        return self.__processPIInfo(self._statusDB.ContactAuthorPI(rList), ['last_name'])

    def __processPIInfo(self, rows, name_items):
        """
        """
        Map = {}
        if not rows:
            return Map
        #
        for dataD in rows:
            dep_id = ''
            if ('id' in dataD) and dataD['id']:
                dep_id = str(dataD['id'])
            #
            if not dep_id:
                continue
            #
            country = ''
            if ('country' in dataD) and dataD['country']:
                country = str(dataD['country'])
            #
            if not country:
                continue
            #
            pi_name = ''
            for item in name_items:
                if (item not in dataD) or (not dataD[item]):
                    continue
                #
                if pi_name:
                    pi_name += ' '
                #
                pi_name += str(dataD[item])
            #
            if not pi_name:
                continue
            #
            if dep_id in Map:
                Map[dep_id]['pi_name'] += ", <br/>" + pi_name
                Map[dep_id]['country'] += ", <br/>" + country
                Map[dep_id]['pi_name_only'] += ", <br/>" + pi_name
                Map[dep_id]['pi_country_only'] += ", <br/>" + country
            else:
                Map[dep_id] = {'pi_name': pi_name, 'country': country, 'pi_name_only': pi_name, 'pi_country_only': country}
            #
        #
        return Map

    def __getOrder(self, order, num_rows, order_condition, dataD):
        """
        """
        if not order_condition:
            return str(order)
        #
        condition_list = order_condition.split(':')
        value_list = condition_list[1].split(',')
        num = len(value_list) + 1
        if (condition_list[0] in dataD) and dataD[condition_list[0]]:
            count = 1
            for value in value_list:
                if condition_list[0] == 'dep_notify':
                    if dataD[condition_list[0]].find(value) != -1:
                        num = count
                        break
                    #
                elif dataD[condition_list[0]] == value:
                    num = count
                    break
                #
                count += 1
            #
        #
        return str(num * num_rows + order)

    def __processLockLabelForCommunication(self, dataDict):
        """
        """
        # Determine if unlock from communication is allowed when invoked from WFM
        # dep_status_code and dep_locking must be present.
        # Must be unlocked
        # Must not be DEP, OBS, or WDRN status (author initiated post_relase allows REL if pdb_id present (i.e. not map onlu)
        if ('dep_status_code' not in dataDict) or ('dep_locking' not in dataDict) or (str(dataDict['dep_locking']).upper() == 'WFM') or \
           (str(dataDict['dep_status_code']).upper() in ('DEP', 'OBS', 'WDRN')):
            return ''

        # Check for map only
        if str(dataDict['dep_status_code']).upper() == 'REL' and ('pdb_id' not in dataDict or len(dataDict['pdb_id']) < 2):
            return ""
        return '&allowunlock=yes'

    def __processStatusCode(self, dataDict):
        """
        """
        statusCode = ''
        authorReleaseStatusCode = ''
        titleEM = ''
        authorListEM = ''
        baseStatusCode = ''
        if 'dep_status_code' in dataDict and dataDict['dep_status_code']:
            baseStatusCode = dataDict['dep_status_code']
            if 'dep_post_rel_status' in dataDict and dataDict['dep_post_rel_status']:
                baseStatusCode = dataDict['dep_post_rel_status'] + '(' + baseStatusCode + ')'

        if dataDict['emdb_id'] != '-':
            # EMDB ID
            if dataDict['pdb_id'] != '-':
                # PDB ID present
                if ('dep_status_code' in dataDict) and dataDict['dep_status_code'] and ('dep_status_code_emdb' in dataDict) and dataDict['dep_status_code_emdb']:
                    statusCode = baseStatusCode + '/' + dataDict['dep_status_code_emdb']
                else:
                    statusCode = baseStatusCode
                #
                if ('dep_author_release_status_code' in dataDict) and dataDict['dep_author_release_status_code'] and \
                   ('dep_author_release_status_code_emdb' in dataDict) and dataDict['dep_author_release_status_code_emdb']:
                    authorReleaseStatusCode = dataDict['dep_author_release_status_code'] + '/' + dataDict['dep_author_release_status_code_emdb']
                elif ('dep_author_release_status_code' in dataDict) and dataDict['dep_author_release_status_code']:
                    authorReleaseStatusCode = dataDict['dep_author_release_status_code']
                #
            else:
                # Map only
                if ('dep_status_code_emdb' in dataDict) and dataDict['dep_status_code_emdb']:
                    statusCode = dataDict['dep_status_code_emdb']
                #
                if ('dep_author_release_status_code_emdb' in dataDict) and dataDict['dep_author_release_status_code_emdb']:
                    authorReleaseStatusCode = dataDict['dep_author_release_status_code_emdb']
                #
                if ('title_emdb' in dataDict) and dataDict['title_emdb']:
                    titleEM = dataDict['title_emdb']
                #
                if ('author_list_emdb' in dataDict) and dataDict['author_list_emdb']:
                    authorListEM = dataDict['author_list_emdb']
                #
            #
        else:
            # No emdb id
            statusCode = baseStatusCode
            #
            if ('dep_author_release_status_code' in dataDict) and dataDict['dep_author_release_status_code']:
                authorReleaseStatusCode = dataDict['dep_author_release_status_code']
            #
        #
        return statusCode, authorReleaseStatusCode, titleEM, authorListEM

    def __getGroupStatusCode(self, entryIdList):
        status_code = 'unknown'
        count = 0
        info_rows = self._statusDB.getSimpleEntryInfo(depositionids=entryIdList)
        if not info_rows:
            return status_code
        #
        statusMap = {}
        for dataD in info_rows:
            if ('status_code' not in dataD) or (not dataD['status_code']):
                continue
            #
            code = str(dataD['status_code']).strip().upper()
            if code in statusMap:
                statusMap[code] += 1
            else:
                statusMap[code] = 1
            #
        #
        for code, val in statusMap.items():
            if val > count:
                count = val
                status_code = code
            #
        #
        return status_code



##
# File:  PdbFileGenerator.py
# Date:  24-Apr-2017
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys

from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class PdbFileGenerator(BaseClass):
    def __init__(self, reqObj=None, entryList=None, verbose=False, log=sys.stderr):
        """
        """
        super(PdbFileGenerator, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        self.__entryList = entryList

    def run(self):
        """
        """
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runSingle(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __runSingle(self, entry_id):
        message, modelFile = self._getExistingArchiveFile(entry_id, 'model', 'pdbx', 'latest')
        if message:
            self._dumpPickle(entry_id + "_PdbFileGenerator", message)
            return
        #
        message, pdbFile = self.__generatePdbFile(entry_id, modelFile)
        if message:
            self._dumpPickle(entry_id + "_PdbFileGenerator", message)
            return
        #
        archivePdbFile = self._findArchiveFileName(entry_id, 'model', 'pdb', 'next')
        message = self._copyFileUtil(pdbFile, archivePdbFile)
        if message:
            self._dumpPickle(entry_id + "_PdbFileGenerator", message)
        else:
            self._dumpPickle(entry_id + "_PdbFileGenerator", 'OK')
        #

    def __generatePdbFile(self, entry_id, inputFile):
        pdbFile = entry_id + "_PdbFileGenerator.pdb"
        self._removeFile(os.path.join(self._sessionPath, pdbFile))
        logFile = 'PdbFileGenerator_generate_pdb_' + entry_id + '.log'
        clogFile = 'PdbFileGenerator_generate_pdb_command_' + entry_id + '.log'
        cmd = self._getCmd('${BINPATH}/maxit', inputFile, pdbFile, logFile, clogFile, ' -o 2 ')
        self._runCmd(cmd)
        if os.access(os.path.join(self._sessionPath, pdbFile), os.F_OK):
            return '', os.path.join(self._sessionPath, pdbFile)
        #
        return "Convert PDB file failed.", ""

    def __getReturnMessage(self):
        message = ''
        for entry_id in self.__entryList:
            pickleData = self._loadPickle(entry_id + "_PdbFileGenerator")
            if pickleData and pickleData != 'OK':
                message += "Generate PDB file for " + entry_id + " failed:\n\t" + pickleData + "\n"
            else:
                message += "Generate PDB file for " + entry_id + " successfully.\n"
            #
            self._removePickle(entry_id + "_PdbFileGenerator")
        #
        return message


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002030")
    myReqObj.setValue("sessionid", "88626e0cc0b1a1bbd10bb2df8a0d68573fcbd5fe")
    myentryList = ['D_8000210285', 'D_8000210286']
    pfGenUtil = PdbFileGenerator(reqObj=myReqObj, entryList=myentryList, verbose=False, log=sys.stderr)
    print(pfGenUtil.run())



##
# File:  AnnotAssignUtil.py
# Date:  09-June-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import os
import sys

from wwpdb.io.file.mmCIFUtil import mmCIFUtil
from wwpdb.io.locator.PathInfo import PathInfo


class AnnotAssignUtil(object):
    """
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__lfh = log
        self.__verbose = verbose
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)

    def updateAnnotatorAssignment(self, assignList=None):
        """
        """
        if not assignList:
            return
        #
        for alist in assignList:
            sourcePath = self.__pI.getFilePath(dataSetId=alist[0], wfInstanceId=None, contentType='model', formatType='pdbx',
                                               fileSource='archive', versionId='latest', partNumber='1')
            if not sourcePath or not os.access(sourcePath, os.F_OK):
                continue
            #
            cifObj = mmCIFUtil(filePath=sourcePath)
            cifObj.UpdateSingleRowValue('pdbx_database_status', 'pdbx_annotator', 0, alist[1].upper())
            cifObj.WriteCif(outputFilePath=sourcePath)
        #



##
# File:  SequenceMerger.py
# Date:  30-May-2017
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2017 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys

from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class SequenceMerger(BaseClass):
    def __init__(self, reqObj=None, entryList=None, templateFile=None, verbose=False, log=sys.stderr):
        """
        """
        super(SequenceMerger, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        self.__entryList = entryList
        self.__templateFile = templateFile

    def run(self):
        """
        """
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runSingle(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __runSingle(self, entry_id):
        message, modelFile = self._getExistingArchiveFile(entry_id, 'model', 'pdbx', 'latest')
        if message:
            self._dumpPickle(entry_id + "_SequenceMerger", message)
            return
        #
        message, updatedModelFile = self.__updateModelFile(entry_id, modelFile)
        if message:
            self._dumpPickle(entry_id + "_SequenceMerger", message)
            return
        #
        archiveModelFile = self._findArchiveFileName(entry_id, 'model', 'pdbx', 'next')
        message = self._copyFileUtil(updatedModelFile, archiveModelFile)
        if message:
            self._dumpPickle(entry_id + "_SequenceMerger", message)
        else:
            self._dumpPickle(entry_id + "_SequenceMerger", 'OK')
        #

    def __updateModelFile(self, entry_id, inputFile):
        updatedModelFile = entry_id + "_SequenceMerger.cif"
        self._removeFile(os.path.join(self._sessionPath, updatedModelFile))
        logFile = 'SequenceMerger_update_cif_' + entry_id + '.log'
        clogFile = 'SequenceMerger_update_cif_command_' + entry_id + '.log'
        option = ' -example ' + self.__templateFile
        mismatch_flag = str(self._reqObj.getValue("mismatch_flag"))
        if mismatch_flag:
            option += ' -rename '
        #
        cmd = self._getCmd('${BINPATH}/MergePolySeqInfo', inputFile, updatedModelFile, logFile, clogFile, option)
        self._runCmd(cmd)
        #
        msg = self._getLogMessage(os.path.join(self._sessionPath, logFile))
        cmsg = self._getLogMessage(os.path.join(self._sessionPath, clogFile))
        if cmsg:
            if msg:
                msg += '\n'
            #
            msg += cmsg
        #
        if msg:
            return msg, ""
        #
        if os.access(os.path.join(self._sessionPath, updatedModelFile), os.F_OK):
            return '', os.path.join(self._sessionPath, updatedModelFile)
        #
        return "Merge sequence information failed.", ""

    def __getReturnMessage(self):
        message = ''
        for entry_id in self.__entryList:
            pickleData = self._loadPickle(entry_id + "_SequenceMerger")
            if pickleData and pickleData != 'OK':
                message += "Merge sequence information for " + entry_id + " failed:\n\t" + pickleData + "\n"
            else:
                message += "Merge sequence information for " + entry_id + " successfully.\n"
            #
            self._removePickle(entry_id + "_SequenceMerger")
        #
        return message


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002030")
    myReqObj.setValue("sessionid", "fc3d934eb200f2f817eb1a8f2c608640cd3b1ba1")
    myEntryList = ['D_8000210285', 'D_8000210286']
    tempFile = '/wwpdb_da/da_top/data_test/archive/D_8000210285/D_8000210285_model_P1.cif.V10'
    pfGenUtil = SequenceMerger(reqObj=myReqObj, entryList=myEntryList, templateFile=tempFile, verbose=False, log=sys.stderr)
    print(pfGenUtil.run())



##
# File:  StatusDbApi.py
# Date:  04-May-2015
# Updates:
##
"""
Providing addintaional APIs for WFE to get info from status database.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.wf.dbapi.WFEtime import getTimeNow
from wwpdb.apps.workmanager.db_access.DbApiUtil import DbApiUtil


class StatusDbApi(object):
    __schemaMap = { "AUTENTICATE" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.user_name = '%s' and u.password = '%s' and u.active = 0",
                    "SELECT_USER" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.user_name = '%s'",
             "SELECT_ACTIVE_USER" : "select user_name, first_name, last_name, initials from da_users as dau, da_group as dag where " +
                                    "dau.da_group_id=dag.da_group_id and code='ANN' and site in ('PDBj', 'RCSB', 'PDBC') " +
                                    "and initials not in ( 'JY', 'JW') and active = 0",
              "SELECT_USER_EMAIL" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.email = '%s'",
            "SELECT_USER_INITIAL" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.initials = '%s'",
                    "UPDATE_USER" : "update da_users set password = '%s', email = '%s', first_name = '%s', last_name = '%s' where " +
                                    "user_name = '%s'",
                   "SELECT_SITES" : "select g.code, g.group_name, g.site, g.da_group_id, g.main_page from da_group as g",
                "SELECT_SITE_ANN" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.active = 0 and g.code = '%s' and g.site = '%s'",
              "SELECT_GROUP_USER" : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = " +
                                    "g.da_group_id and u.active = 0 and u.da_group_id = '%s'",
              "SELECT_SITE_GROUP" : "select code, group_name, site, da_group_id group_id from da_group where site = '%s'", 
    "SELECT_SITE_GROUP_WITH_CODE" : "select code, group_name, site, da_group_id group_id from da_group where site = '%s' and code = '%s'",
              "SELECT_SITE_USER"  : "select u.user_name, u.password, u.da_group_id group_id, u.email, u.initials, u.first_name, u.last_name, " +
                                    "u.active, g.code, g.group_name, g.site from da_users as u, da_group as g where u.da_group_id = g.da_group_id " +
                                    "and g.site = '%s' order by u.active, g.code, u.initials",
              "SERVER_MONITORING" : "select hostname, status_timestamp from engine_monitoring",
        "SELECT_DEPOSITION_BY_ID" : "select depPW as deppw, pdb_id, bmrb_id, emdb_id, title from deposition where dep_set_id = '%s'",
          "SELECT_TIMESTAMP_INFO" : "select ordinal, mtime, event, info1, info2 from timestamp where dep_set_id = '%s' order by ordinal",
   "SELECT_SINGLE_ANNO_SELECTION" : "select dep_set_id, annotator_initials from anno_selection where dep_set_id = '%s'",
 "SELECT_MULTIPLE_ANNO_SELECTION" : "select dep_set_id, annotator_initials from anno_selection where dep_set_id in ( '%s' )",
                "GET_CLASS_BY_ID" : "select wf_class_id, wf_class_name, title, author, version, class_file from wf_class_dict where " +
                                    "wf_class_id = '%s'",
          "DELETE_ANNO_SELECTION" : "delete from anno_selection where dep_set_id = '%s'",
          "INSERT_ANNO_SELECTION" : "insert into anno_selection (dep_set_id, annotator_initials) values ( '%s', '%s' )",
#                "UP_INST_STATUS" : "update wf_instance set inst_status = '%s', owner = '%s' where dep_set_id = '%s' and wf_inst_id = '%s' " +
#                                   " and wf_class_id = '%s'",
    "SELECT_SINGLE_MESSAGE_TRACK" : "select dep_set_id, major_issue, last_reminder_sent_date, last_validation_sent_date, last_message_sent_date, " +
                                    "last_message_received_date from remind_message_track where dep_set_id = '%s'",
  "SELECT_MULTIPLE_MESSAGE_TRACK" : "select dep_set_id, major_issue, last_reminder_sent_date, last_validation_sent_date, last_message_sent_date, " +
                                    "last_message_received_date from remind_message_track where dep_set_id in ( '%s' )",
           "DELETE_MESSAGE_TRACK" : "delete from remind_message_track where dep_set_id = '%s'",
          "UPDATE_ANN_DEPOSITION" : "update deposition set annotator_initials  = '%s' where dep_set_id = '%s'",
           "UPDATE_ANN_LAST_INST" : "update dep_last_instance set annotator_initials  = '%s' where dep_set_id = '%s'",
           "SELECT_LAST_INSTANCE" : "select class_id as wf_class_id, inst_id as wf_inst_id, inst_status, dep_set_id, dep_exp_method, pdb_id, dep_bmrb_id as " +
                                    "bmrb_id, dep_emdb_id as emdb_id, dep_status_code, dep_status_code_exp, dep_author_release_status_code, " +
                                    "dep_initial_deposition_date, annotator_initials, dep_notify, dep_locking, dep_title, dep_author_list, dep_post_rel_status from " +
                                    "dep_last_instance where dep_set_id = '%s'", 
        "SELECT_WF_LAST_INSTANCE" : "select ordinal, wf_inst_id, wf_class_id, dep_set_id, owner, inst_status, status_timestamp from wf_instance " +
                                    "where dep_set_id = '%s' and wf_class_id = '%s' order by status_timestamp desc limit 1",
         "SELECT_WF_ALL_INSTANCE" : "select wf_inst_id, wf_class_id, dep_set_id, inst_status, status_timestamp from wf_instance " +
                                    "where dep_set_id = '%s' and wf_class_id not in ( 'Annotate', 'depUpload' ) order by wf_inst_id",
           "SELECT_COMMUNICATION" : "select ordinal, sender, receiver, dep_set_id, wf_class_id, wf_inst_id, wf_class_file, command, status, actual_timestamp, " +
                                    "parent_dep_set_id, parent_wf_class_id, parent_wf_inst_id, data_version from communication where " +
                                    "parent_dep_set_id = '%s' order by actual_timestamp desc limit 1",
           "SELECT_DEP_WF_STATUS" : "select inst_status from dep_instance where dep_set_id = '%s' and inst_id = '%s' and class_id = '%s'",
                  "GET_REAL_FLOW" : "select wf_task_id, task_status, status_timestamp, task_type from wf_task where dep_set_id = '%s' and wf_inst_id = '%s' " +
                                    "and wf_class_id = '%s' order by status_timestamp asc",
           "CONTACT_AUTHOR_PI"    : "select dep_set_id as id, email, last_name, role, country from user_data where dep_set_id in ( '%s' ) and role = '%s'",
           "CHECK_TABLE_EXIST"    : "select distinct table_name from  information_schema.tables where table_schema = '%s' and table_name = '%s'",
                          "COUNT" : "select count(*) from %s",
                 "GET_ENTRY_LIST" : "select dep_set_id,pdb_id,emdb_id,bmrb_id from deposition where %s",
      "GET_ENTRY_LIST_FROM_GROUP" : "select group_id, dep_set_id from group_deposition_information where group_id in ( '%s' ) order by dep_set_id",
                "SELECT_GROUP_ID" : "select dep_set_id, group_id from group_deposition_information where dep_set_id in ( '%s' )",
                 "GET_ENTRY_INFO" : "select dep_set_id, initial_deposition_date, annotator_initials, status_code from deposition where dep_set_id in ( '%s' ) " +
                                    "order by dep_set_id",
                   }
    #
    __comm_items = ['sender', 'receiver', 'dep_set_id', 'wf_class_id', 'wf_inst_id', 'wf_class_file', 'command', 'status', 'actual_timestamp',
                    'parent_dep_set_id', 'parent_wf_class_id', 'parent_wf_inst_id', 'data_version']
    """
    """
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        """
        """
        self.__lfh = log
        self.__verbose = verbose
        self.__siteId = siteId
        self.__cI = ConfigInfo(self.__siteId)
        self.__dbServer = self.__cI.get("SITE_DB_SERVER")
        self.__dbHost = self.__cI.get("SITE_DB_HOST_NAME")
        self.__dbName = self.__cI.get("SITE_DB_DATABASE_NAME")
        self.__dbUser = self.__cI.get("SITE_DB_USER_NAME")
        self.__dbPw = self.__cI.get("SITE_DB_PASSWORD")
        self.__dbSocket = self.__cI.get("SITE_DB_SOCKET")
        self.__dbPort = int(self.__cI.get("SITE_DB_PORT_NUMBER"))
        #
        self.__dbApi = DbApiUtil(dbServer=self.__dbServer, dbHost=self.__dbHost, dbName=self.__dbName, dbUser=self.__dbUser, dbPw=self.__dbPw,
                                 dbSocket=self.__dbSocket, dbPort=self.__dbPort, verbose=self.__verbose, log=self.__lfh)
        self.__dbApi.setSchemaMap(self.__schemaMap)

    def __getDataDir(self, key, parameter, idx):
        dlist = self.__dbApi.selectData(key=key, parameter=parameter)
        if dlist:
            return dlist[idx]
        #
        return None

    def Autenticate(self, username=None, password=None):
        if not username or not password:
            return None
        #
        return self.__getDataDir("AUTENTICATE", (username, password), 0)

    def getUserByName(self, username=None):
        if not username:
            return None
        #
        return self.__getDataDir("SELECT_USER", (username), 0)

    def getActiveAnnoList(self):
        return self.__dbApi.selectData(key="SELECT_ACTIVE_USER", parameter=())

    def getUserByEmail(self, email=None):
        if not email:
            return None
        #
        return self.__getDataDir("SELECT_USER_EMAIL", (email), 0)

    def getUserByInitial(self, initial=None):
        if not initial:
            return None
        #
        return self.__getDataDir("SELECT_USER_INITIAL", (initial), 0)

    def updateUser(self, password=None, email=None, first_name=None, last_name=None, user_name=None):
        if not password or not email or not first_name or not last_name or not user_name:
            return 'Update user information failed.'
        #
        sql = self.__schemaMap['UPDATE_USER'] % (password, email, first_name, last_name, user_name)
        ret = self.__dbApi.runUpdateSQL(sql)
        if ret != 'OK':
            return 'Update user information failed.'
        else:
            return 'User Information Updated.'
        #

    def getAnnUser(self, site=None):
        if not site:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_SITE_ANN", parameter=('ANN', site))

    def getAnnLeader(self, site=None):
        if not site:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_SITE_ANN", parameter=('LANN', site))

    def getGroupUser(self, da_group_id=None):
        if not da_group_id:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_GROUP_USER", parameter=(da_group_id))

    def getSites(self):
        """Returns list of sites in WFM user table"""
        return self.__dbApi.selectData(key="SELECT_SITES", parameter=())

    def getSiteGroup(self, site=None):
        if not site:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_SITE_GROUP", parameter=(site))

    def getSiteGroupWithCode(self, site=None, code=None):
        if not site or not code:
            return None
        #
        return self.__getDataDir("SELECT_SITE_GROUP_WITH_CODE", (site, code), 0)

    def getSiteUser(self, site=None):
        if not site:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_SITE_USER", parameter=(site))

    def getDepInfo(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__getDataDir("SELECT_DEPOSITION_BY_ID", (depositionid), 0)

    def getServerInfo(self):
        return self.__dbApi.selectData(key="SERVER_MONITORING", parameter=())

    def getTimeStampInfo(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_TIMESTAMP_INFO", parameter=(depositionid))

    def getLastInstance(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__getDataDir("SELECT_LAST_INSTANCE", (depositionid), 0)

    def runSelectSQL(self, sql):
        return self.__dbApi.runSelectSQL(sql)

    def killWorkFlow(self, depositionid=None):
        if not depositionid:
            return 'No deposition ID defined.'
        #
        cloumns = {}
        cloumns['command'] = 'killWF'
        cloumns['actual_timestamp'] = str(getTimeNow())
        cloumns['receiver'] = 'WFE'
        cloumns['status'] = 'PENDING'
        set_command = self.__getSetCommand(cloumns)
        sql = 'update communication set ' + set_command + " where dep_set_id = '" + str(depositionid) + "'"
        ret = self.__dbApi.runUpdateSQL(sql)
        if ret != 'OK':
            return 'Failed to mark ' + depositionid + ' for process kill'
        else:
            return 'OK'
        #

    def addToMyList(self, depositionid=None, initial=None):
        if not depositionid or not initial:
            return
        #
        if self.__dbApi.selectData(key="SELECT_SINGLE_ANNO_SELECTION", parameter=(depositionid)):
            return
        #
        self.__dbApi.runUpdateSQLwithKey(key="INSERT_ANNO_SELECTION", parameter=(depositionid, initial))

    def removeFromMyList(self, depositionid=None):
        if not depositionid:
            return
        #
        if not self.__dbApi.selectData(key="SELECT_SINGLE_ANNO_SELECTION", parameter=(depositionid)):
            return
        #
        self.__dbApi.runUpdateSQLwithKey(key="DELETE_ANNO_SELECTION", parameter=(depositionid))

    def getAnnoSelection(self, depositionids=None):
        return self.__getSelectionResult(depositionids, 'SELECT_MULTIPLE_ANNO_SELECTION')

    def getWfClassByID(self, classID=None):
        if not classID:
            return None
        #
        return self.__getDataDir("GET_CLASS_BY_ID", (classID), 0)

    def getSimpleEntryInfo(self, depositionids=None):
        return self.__getSelectionResult(depositionids, 'GET_ENTRY_INFO')

    #   def updateInstStatus(self, status=None, owner=None, depositionid=None, instanceid=None, classID=None):
    #       if not status or not owner or not depositionid or not instanceid or not classID:
    #           return 'Start Annotate workflow failed.'
    #       #
    #       sql = self.__schemaMap['UP_INST_STATUS'] % ( status, owner, depositionid, instanceid, classID)
    #       ret = self.__dbApi.runUpdateSQL(sql)
    #       if ret != 'OK':
    #           return 'Start Annotate workflow failed.'
    #       else:
    #           return 'OK'
    #       #

    def getRemindMessageTrack(self, depositionids=None):
        return self.__getSelectionResult(depositionids, 'SELECT_MULTIPLE_MESSAGE_TRACK')

    def insertRemindMessageTrack(self, depositionid=None, dataMap=None):
        """
        """
        if not depositionid or not dataMap:
            return
        #
        if self.__dbApi.selectData(key="SELECT_SINGLE_MESSAGE_TRACK", parameter=(depositionid)):
            self.__dbApi.runUpdateSQLwithKey(key="DELETE_MESSAGE_TRACK", parameter=(depositionid))
        #
        key = []
        values = []
        key.append('dep_set_id')
        values.append("'" + depositionid + "'")
        for k, v in dataMap.items():
            key.append(k)
            values.append("'" + v + "'")
        #
        sql = 'insert into remind_message_track ( ' + ', '.join(key) + ' ) values ( ' + ', '.join(values) + ' ) '
        self.__dbApi.runUpdateSQL(sql)

    def updateAnnotatorAssignment(self, assignList=None):
        """
        """
        if not assignList:
            return
        #
        for alist in assignList:
            sql = self.__schemaMap['UPDATE_ANN_DEPOSITION'] % (alist[1], alist[0])
            _ret = self.__dbApi.runUpdateSQL(sql)  # noqa: F841
            # if rows < 1:
            #   catch error
            sql = self.__schemaMap['UPDATE_ANN_LAST_INST'] % (alist[1], alist[0])
            _ret = self.__dbApi.runUpdateSQL(sql)  # noqa: F841
            # if rows < 1:
            #   catch error
        #

    def getLastWFInstance(self, depositionid=None, classid=None):
        if not depositionid or not classid:
            return None
        #
        return self.__getDataDir("SELECT_WF_LAST_INSTANCE", (depositionid, classid), -1)

    def getAllWFInstances(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__dbApi.selectData(key="SELECT_WF_ALL_INSTANCE", parameter=(depositionid))

    def getLastWFCommunication(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__getDataDir("SELECT_COMMUNICATION", (depositionid), -1)

    def getDepositionStatus(self, depositionid=None, instid=None, classid=None):
        status = 'unkown'
        if not depositionid or not classid or not instid:
            return status
        #
        data = self.__getDataDir("SELECT_DEP_WF_STATUS", (depositionid, instid, classid), 0)
        if data:
            status = data['inst_status'].upper()
        #
        return status

    def getRealFlow(self, depositionid=None, instid=None, classid=None):
        if not depositionid or not classid or not instid:
            return None
        #
        return self.__dbApi.selectData(key="GET_REAL_FLOW", parameter=(depositionid, instid, classid))

    def ContactAuthorPI(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__dbApi.selectData(key="CONTACT_AUTHOR_PI", parameter=("', '".join(depositionid), 'principa'))

    def ValidContactAuthor(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__getDataDir("CONTACT_AUTHOR_PI", (depositionid, 'valid'), 0)

    def getCommandStatus(self, depositionid=None):
        status = 'unkown'
        if not depositionid:
            return status
        #
        data = self.getLastWFCommunication(depositionid=depositionid)
        if data and ('status' in data):
            status = data['status'].upper()
        #
        return status

    def insertCommunicationCommand(self, depositionid=None, instid=None, classid=None, command=None, classname=None, dataversion=None):
        if not depositionid or not classid or not command or not classname:
            return 'Start Annotate workflow failed.'
        #
        myD = {}
        myD['sender'] = 'WFM'
        myD['receiver'] = 'WFE'
        myD['dep_set_id'] = depositionid
        myD['wf_class_file'] = classname
        myD['command'] = command
        myD['status'] = 'pending'
        myD['parent_dep_set_id'] = depositionid
        myD['parent_wf_class_id'] = classid
        if instid:
            myD['parent_wf_inst_id'] = instid
        #
        if dataversion:
            myD['data_version'] = dataversion
        #
        sql = ''
        comm = self.getLastWFCommunication(depositionid=depositionid)
        if comm:
            set_comm = ''
            for item in self.__comm_items:
                if set_comm:
                    set_comm += ', '
                #
                if item in myD:
                    set_comm += item + " = '" + str(myD[item]) + "'"
                else:
                    set_comm += item + ' = Null'
                #
            #
            if set_comm:
                sql = 'update communication set ' + set_comm + " where ordinal = '" + str(comm['ordinal']) + "'"
            #
        else:
            items = ''
            values = ''
            for item in self.__comm_items:
                if item not in myD:
                    continue
                #
                if items:
                    items += ', '
                    values += ', '
                #
                items += item
                values += "'" + str(myD[item]) + "'"
            #
            if items and values:
                sql = 'insert communication ( ' + items + ' ) values ( ' + values + ' )'
            #
        #
        if not sql:
            return 'Start Annotate workflow failed.'
        #
        ret = self.__dbApi.runUpdateSQL(sql)
        if ret != 'OK':
            return 'Start Annotate workflow failed.'
        else:
            return 'OK'
        #

    def runUpdate(self, table=None, where=None, data=None):
        return self.__dbApi.runUpdate(table=table, where=where, data=data)

    def isTableExist(self, table=None):
        if not table:
            return False
        #
        ret_list = self.__dbApi.selectData(key="CHECK_TABLE_EXIST", parameter=(self.__dbName, table))
        if len(ret_list) > 0:
            return True
        #
        return False

    def isTableValid(self, table=None):
        if not table:
            return False
        #
        if self.isTableExist(table=table):
            rdir = self.__getDataDir("COUNT", (table), 0)
            if rdir['count(*)'] > 0:
                return True
            #
        #
        return False

    def getEntryListForGroup(self, groupids=None):
        if not groupids:
            return None
        #
        # return self.__dbApi.selectData(key="GET_ENTRY_LIST_FROM_GROUP", parameter=(groupid))
        return self.__getSelectionResult(groupids, 'GET_ENTRY_LIST_FROM_GROUP')

    def getGroupIds(self, depositionids=None):
        return self.__getSelectionResult(depositionids, 'SELECT_GROUP_ID')

    def __getSetCommand(self, data):
        command = ''
        for k, v in data.items():
            if command:
                command += ', '
            #
            command += k + " = '" + v + "'"
        #
        return command

    def __getSelectionResult(self, depositionids, schema_key):
        if not depositionids:
            return None
        #
        ids = "', '".join(depositionids)
        sql = self.__schemaMap[schema_key] % (ids)
        #
        return self.__dbApi.runSelectSQL(sql)

    def getDistinctAnnotatorInitials(self):
        aiList = []
        rows = self.__dbApi.runSelectSQL('select distinct upper(annotator_initials) from deposition')
        for row in rows:
            if ('upper(annotator_initials)' in row) and row['upper(annotator_initials)']:
                aiList.append(row['upper(annotator_initials)'])
            #
        #
        return aiList

    def getActiveAnnotatorInitials(self):
        aaiList = []
        rows = self.__dbApi.runSelectSQL('select upper(initials) from da_users where active = 0')
        for row in rows:
            if ('upper(initials)' in row) and row['upper(initials)']:
                aaiList.append(row['upper(initials)'])
            #
        #
        return aaiList

    def getRetiredAnnotatorInitials(self):
        rtList = []
        distinctList = self.getDistinctAnnotatorInitials()
        activeList = self.getActiveAnnotatorInitials()
        for ai in distinctList:
            if (ai == 'UNKNOWN') or (ai == 'UNASSIGN') or (ai in activeList):
                continue
            #
            rtList.append(ai)
        #
        return rtList

    def getEntryIdListFromInputIdString(self, entry_id_string):
        group_ids = []
        id_type_map = {}
        error_message = ''
        #
        input_id_list = entry_id_string.upper().replace(',', ' ').replace('\n', ' ').replace('\t', ' ').split(' ')
        for input_id in input_id_list:
            if not input_id:
                continue
            #
            id_type = ''
            if input_id[:2] == 'D_':
                id_type = 'dep_set_id'
            elif input_id[:2] == 'G_':
                id_type = 'group_id'
            elif (input_id[:4] == "EMD-") or (input_id[:5] == "EMDB-"):
                id_type = 'emdb_id'
            elif len(input_id) == 4:
                id_type = 'pdb_id'
            elif len(input_id) == 5:
                id_type = 'bmrb_id'
            elif (len(input_id) == 12) and input_id.lower().startswith('pdb_'):
                id_type = 'pdb_id'
            #
            if not id_type:
                if error_message:
                    error_message += "\n"
                #
                error_message += "'" + input_id + "' is not a valid ID."
                continue
            #
            if id_type == 'group_id':
                group_ids.append(input_id)
            else:
                if id_type in id_type_map:
                    id_type_map[id_type].append(input_id)
                else:
                    id_type_map[id_type] = [input_id]
                #
                if input_id.lower().startswith('pdb_0000') and (id_type == 'pdb_id'):
                    short_pdb_id = input_id[8:]
                    #
                    if id_type in id_type_map:
                        id_type_map[id_type].append(short_pdb_id)
                    else:
                        id_type_map[id_type] = [short_pdb_id]
                    #
                #
            #
        #
        if (not id_type_map) and (not group_ids):
            if not error_message:
                error_message = "No Entry IDs defined."
            #
            return error_message, []
        #
        group_error_message, entryList = self.__getDepIDFromGroupID(group_ids)
        if group_error_message:
            if error_message:
                error_message += "\n"
            #
            error_message += group_error_message
        #
        entry_error_message, other_entry_list = self.__getDepIDFromFromIdTypeMap(id_type_map)
        if entry_error_message:
            if error_message:
                error_message += "\n"
            #
            error_message += entry_error_message
        #
        if other_entry_list:
            entryList.extend(other_entry_list)
            entryList = sorted(set(entryList))
        #
        return error_message, entryList

    def __getDepIDFromGroupID(self, group_ids):
        if not group_ids:
            return '', []
        #
        group_ids = sorted(set(group_ids))
        return self.__processGetEntryListResult(self.getEntryListForGroup(groupids=group_ids), group_ids, ['group_id'])

    def __getDepIDFromFromIdTypeMap(self, id_type_map):
        if not id_type_map:
            return '', []
        #
        parameter = ''
        input_id_list = []
        id_type_list = []
        for id_type in ('dep_set_id', 'pdb_id', 'bmrb_id', 'emdb_id'):
            if (id_type not in id_type_map) or (not id_type_map[id_type]):
                continue
            #
            id_type_map[id_type] = sorted(set(id_type_map[id_type]))
            input_id_list.extend(id_type_map[id_type])
            id_type_list.append(id_type)
            if parameter:
                parameter += " or "
            #
            parameter += " " + id_type + " in ( '" + "', '".join(id_type_map[id_type]) + "' ) "
        #
        if not parameter:
            return '', []
        #
        return self.__processGetEntryListResult(self.__dbApi.selectData(key='GET_ENTRY_LIST', parameter=(parameter)), input_id_list, id_type_list)

    def __processGetEntryListResult(self, return_list, input_id_list, id_type_list):
        return_id_list = []
        found_id_map = {}
        if return_list:
            for myD in return_list:
                if ('dep_set_id' in myD) and myD['dep_set_id']:
                    return_id_list.append(myD['dep_set_id'].upper())
                #
                for id_type in id_type_list:
                    if (id_type in myD) and myD[id_type]:
                        found_id_map[myD[id_type].upper()] = 'yes'
                    #
                #
            #
        #
        error_message = ''
        for input_id in input_id_list:
            if input_id in found_id_map:
                continue
            #
            if error_message:
                error_message += "\n"
            #
            error_message += "'" + input_id + "' is not a valid ID."
        #
        return error_message, return_id_list

    def deleteSite(self, site):
        """Deletes a site by retrieving the group id and then deleting users and group"""

        status = 'OK'
        sitegr = self.getSiteGroup(site)
        for s in sitegr:
            groupid = s['group_id']

            sql = "delete from da_users where da_group_id = '{}'".format(groupid)
            ret = self.__dbApi.runUpdateSQL(sql)

            sql = "delete from da_group where da_group_id = '{}'".format(groupid)
            ret = self.__dbApi.runUpdateSQL(sql)
            if ret != 'OK':
                status = ret

        return status

    def addSite(self, site, lead, email, first, last):  # pylint: disable=unused-argument
        # First determine new group
        sites = self.getSites()
        maxgroup = 0
        status = 'OK'
        for s in sites:
            if s['site'].lower() == site.lower():
                status = 'Site already exists'
                return status
            maxgroup = max(maxgroup, s['da_group_id'])

        for code in ['LANN', 'ANN']:
            maxgroup += 1
            if code == 'LANN':
                group_name = "{} - Lead Annotator".format(site)
                main_page = "RCSBLeadAnnotator.html"
            else:
                group_name = "{} - Annotator".format(site)
                main_page = "Annotators.html"

            data = {'group_name' : group_name, 'site' : site, 'main_page' : main_page}
            ret = status = self.runUpdate(table='da_group', where={'code' : code, 'da_group_id' : str(maxgroup)}, data=data)
            if ret != 'OK':
                status = ret

        r = self.getSiteGroupWithCode(site, 'LANN')
        lann_gr = r['group_id']

        data = {'user_name' : site.upper(), 'password': site.upper(), 'email': email, 'initials': site.upper(), 'first_name': first,
                'last_name': last + "(Lead)"}
        ret = status = self.runUpdate(table='da_users', where={'da_group_id' : str(lann_gr)}, data=data)
        if ret != 'OK':
            status = ret

        return status


if __name__ == '__main__':
    db = StatusDbApi(siteId='WWPDB_DEPLOY_TEST_RU', verbose=True, log=sys.stderr)
    message, tentryList = db.getEntryIdListFromInputIdString("G_1002014, G_10020,D_8000210666,1abc,D_8000210646,D_1000210646")
    print(message)
    print(tentryList)
    # """
    # alList = db.getDistinctAnnotatorInitials()
    # print(alList)
    # aaiList = db.getActiveAnnotatorInitials()
    # print aaiList
    # rtList = db.getRetiredAnnotatorInitials()
    # print rtList
    # depids = [ 'D_8000210373', 'D_8000210372' ]
    # rows = db.getGroupIds([ 'D_8000210373', 'D_8000210372' ])
    # print rows
    # ok = db.isTableValid(table=sys.argv[1])
    # if ok:
    #     print 'Table ' + sys.argv[1] + ' exist.'
    # else:
    #     print 'Table ' + sys.argv[1] + ' does not exist.'
    # dir = db.Autenticate(username='JY', password='JY')
    # print dir
    # dir = db.getUserByName(username='JY')
    # print dir
    # dir = db.getUserByEmail(email='jasmin@rcsb.rutgers.edu')
    # print dir
    # ss = str(dir)
    # print 'ss='+ss
    # list = db.getServerInfo()
    # print list
    # """



##
# File:  DepictWorkFlow.py
# Date:  24-Mar-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys
import traceback
try:
    from urllib.parse import quote as u_quote
except ImportError:
    from urllib import quote as u_quote

from wwpdb.utils.wf.dbapi.WFEtime import getTimeString
from wwpdb.apps.workmanager.depict.DepictBase import DepictBase
from wwpdb.apps.workmanager.file_access.LogFileUtil import LogFileUtil
from wwpdb.apps.workmanager.workflow_access.WorkflowXMLLoader import WorkflowXMLLoader
from wwpdb.io.locator.PathInfo import PathInfo


class DepictWorkFlow(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(DepictWorkFlow, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #
        self.__sObj = self._reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        self.__MaxBox = 7
        self.__MaxBoxTask = 4
        #
        self.__setup()

    def __setup(self):
        """
        """
        self._connectStatusDB()
        self.__classInfo = self._statusDB.getWfClassByID(classID=self._reqObj.getValue("classID"))
        #
        wfloader = WorkflowXMLLoader(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        wfloader.loadWorkFlowXMLFile(self.__classInfo['class_file'])
        #
        self.__wfMetaDataInfo = wfloader.getMetaDataInfo()
        # self.__wfMetaDataInfo['title'] = classInfo['title']
        self.__wfWorkFlowInfo = wfloader.getWorkFlowInfo()
        self.__wfFlow = self.__wfWorkFlowInfo.values()
        #
        self.__interface_tmplt = self._getPageTemplate('interface_tmplt')

    def getLevel2Setting(self):
        """
        """
        dataD = self.__wfMetaDataInfo
        dataD['wf_entry_id'] = self.__wfFlow[0]['taskID']
        dataD['wf_entry_name'] = self.__wfFlow[0]['name']
        dataD['wf_exit_id'] = self.__wfFlow[-1]['taskID']
        dataD['wf_exit_name'] = self.__wfFlow[-1]['name']
        self._dataInfo['workflow_tmplt'] = [dataD]
        #
        self.__download_tmplt = self._getPageTemplate('download_tmplt')  # pylint: disable=attribute-defined-outside-init
        self.__depictLevel2WorkFlow()
        self.__depictRunWorkFlowInfo()

    def getLevel3Setting(self):
        """
        """
        wfD = self.__wfMetaDataInfo
        wfD['title'] = self.__classInfo['title']
        lastWFInstance = self._statusDB.getLastWFInstance(depositionid=self._reqObj.getValue('identifier'), classid=self._reqObj.getValue('classID'))
        if lastWFInstance:
            if ('inst_status' in lastWFInstance) and lastWFInstance['inst_status']:
                wfD['inst_status'] = lastWFInstance['inst_status']
            #
            if ('status_timestamp' in lastWFInstance) and lastWFInstance['status_timestamp']:
                wfD['status_timestamp'] = getTimeString(lastWFInstance['status_timestamp'])
            #
        #
        wfD['task_info'] = self.__depictLevel3TaskInfo()
        return wfD

    def __depictLevel2WorkFlow(self):
        """
        """
        workflow_module_tmplt = self._getPageTemplate('workflow_module_tmplt')
        notdone_module_tmplt = self._getPageTemplate('notdone_module_tmplt')
        single_module_tmplt = self._getPageTemplate('single_module_tmplt')
        single_module_UI_tmplt = self._getPageTemplate('single_module_UI_tmplt')
        #
        work_module = '<tr>\n'
        single_module = '<tr>\n'
        count = 0
        for wf in self.__wfFlow[1:-1]:
            if 'classID' not in wf:
                continue
            #
            if count == self.__MaxBox:
                count = 0
                work_module += '</tr>\n<tr>\n'
                single_module += '</tr>\n<tr>\n'
            #
            myD = self.__initializeMyD()
            myD = self.__expandMyD(myD, wf, ('classID', 'taskID', 'name'))
            myD['inst_status'] = 'notdone'
            myD['instance'] = ''
            lastWFInstance = self._statusDB.getLastWFInstance(depositionid=self._reqObj.getValue('identifier'), classid=wf['classID'])
            myD = self.__expandMyD(myD, lastWFInstance, ('inst_status', 'wf_inst_id'))
            if not myD['inst_status']:
                myD['inst_status'] = 'notdone'
            #
            myD['run_with_ui'] = ''
            if wf['classID'] in ('TransMod', 'LigMod', 'SeqMod' , 'AnnMod'):
                myD['run_with_ui'] = single_module_UI_tmplt % myD
            elif wf['classID'] == 'ValMod':
                groupId = self._reqObj.getValue('group_id')
                if groupId and groupId.startswith('G_'):
                    myD['run_with_ui'] = single_module_UI_tmplt % myD
                #
            #
            myD['open_interface'] = ''
            if myD['inst_status'] == 'notdone':
                work_module += notdone_module_tmplt % myD
            else:
                myD['download_ciffile'] = self.__processDownLoadCifFile(myD, ' | ')
                if myD['inst_status'] == 'waiting':
                    myD['base_url'] = self._processBaseUrl(wf['classID'])
                    myD['style'] = ''
                    myD['open_interface'] = self.__interface_tmplt % myD + '<br/>'
                #
                work_module += workflow_module_tmplt % myD
            #
            single_module += single_module_tmplt % myD
            count += 1
        #
        work_module += '</tr>\n'
        single_module += '</tr>\n'
        self._dataInfo['sub_workflow_module'] = work_module
        self._dataInfo['single_workflow_module'] = single_module

    def __depictRunWorkFlowInfo(self):
        """
        """
        wf_list = self._statusDB.getAllWFInstances(depositionid=self._reqObj.getValue('identifier'))
        if not wf_list:
            return
        #
        run_module_tmplt = self._getPageTemplate('run_module_tmplt')
        #
        contents = '<tr>\n'
        count = 0
        hasValue = False
        for dir in wf_list:  # pylint: disable=redefined-builtin
            if ('inst_status' in dir) and dir['inst_status'] == 'aborted':
                continue
            #
            hasValue = True
            if count == self.__MaxBox:
                count = 0
                contents += '</tr>\n<tr>\n'
            #
            myD = self.__initializeMyD()
            myD = self.__expandMyD(myD, dir, ('inst_status', 'wf_inst_id', 'wf_class_id', 'status_timestamp'))
            myD['download_ciffile'] = self.__processDownLoadCifFile(myD, '')
            contents += run_module_tmplt % myD
            #
            count += 1
        #
        if not hasValue:
            return
        #
        contents += '</tr>\n'
        self._dataInfo['run_workflow_module'] = contents

    def __depictLevel3TaskInfo(self):
        """
        """
        realFlowList = self._statusDB.getRealFlow(depositionid=self._reqObj.getValue('identifier'), instid=self._reqObj.getValue('instance'),
                                                  classid=self._reqObj.getValue('classID'))
        if not realFlowList:
            return ''
        #
        task_tmplt = self._getPageTemplate('task_tmplt')
        log_tmplt = self._getPageTemplate('download_task_log_tmplt')
        #
        contents = '<tr>\n'
        count = 0
        for dir in realFlowList:  # pylint: disable=redefined-builtin
            if count == self.__MaxBoxTask:
                count = 0
                contents += '</tr>\n<tr>\n'
            #
            myD = self.__initializeMyD()
            myD = self.__expandMyD(myD, dir, ('wf_task_id', 'task_status', 'task_type', 'status_timestamp'))
            #
            wf = self.__wfWorkFlowInfo[myD['wf_task_id']]
            myD = self.__expandMyD(myD, wf, ('name', 'description', 'reference'))
            #
            myD['log_file'] = self.__getTaskLogFile(log_tmplt, myD)
            #
            if myD['task_status'] == 'waiting':
                myD['base_url'] = self._processBaseUrl(myD['classID'])
                myD['style'] = 'style ="color:black;display:block;"'
                myD['open_interface'] = self.__interface_tmplt % myD
            else:
                myD['open_interface'] = ''
            #
            contents += task_tmplt % myD
            count += 1
        #
        contents += '</tr>\n'
        return contents

    def __initializeMyD(self):
        """
        """
        myD = {}
        for item in ('identifier', 'sessionid', 'annotator', 'method', 'urlmethod', 'instance', 'classID'):
            myD[item] = self._reqObj.getValue(item)
        #
        if myD['method'] and (not myD['urlmethod']):
            myD['urlmethod'] = u_quote(myD['method'])
        #
        return myD

    def __expandMyD(self, myD, sourceD, itemTuple):
        """
        """
        for item in itemTuple:
            myItem = item
            if item == 'wf_inst_id':
                myItem = 'instance'
            elif item == 'wf_class_id':
                myItem = 'classID'
            #
            if sourceD and (item in sourceD) and sourceD[item]:
                if item == 'status_timestamp':
                    myD[myItem] = getTimeString(sourceD[item])
                else:
                    myD[myItem] = str(sourceD[item])
            else:
                myD[myItem] = ''
            #
        #
        return myD

    def __processDownLoadCifFile(self, myD, delimiter):
        """
        """
        try:
            pI = PathInfo(siteId=self._siteId, sessionPath=self.__sessionPath, verbose=self._verbose, log=self._lfh)
            filePath = pI.getFilePath(dataSetId=myD['identifier'], wfInstanceId=myD['instance'], contentType='model',
                                      formatType='pdbx', fileSource='wf-instance', versionId='latest', partNumber='1')

            if filePath and os.access(filePath, os.F_OK):
                return delimiter + self.__download_tmplt % myD
            #
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self._lfh)
        #
        return ''

    def __getTaskLogFile(self, log_tmplt, myD):
        """
        """
        self._reqObj.setValue('taskID', myD['wf_task_id'])
        self._reqObj.setValue('reference', myD['reference'])
        logUtil = LogFileUtil(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
        logFilePath = logUtil.getLogFile()
        if logFilePath:
            return log_tmplt % myD
        #
        return ''
